/*
 * Copyright 1999,2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.webdav.methods;

import net.sf.webdav.exceptions.AccessDeniedException;
import net.sf.webdav.exceptions.ObjectAlreadyExistsException;
import net.sf.webdav.exceptions.WebdavException;
import net.sf.webdav.exceptions.ObjectNotFoundException;
import net.sf.webdav.Transaction;
import net.sf.webdav.WebdavStatus;
import net.sf.webdav.ResourceLocks;
import net.sf.webdav.WebdavStore;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Hashtable;
import java.io.IOException;

public class DoDelete extends ReportingMethod {

    private WebdavStore store;
    private ResourceLocks resourceLocks;
    private boolean readOnly;
    private int debug;


    public DoDelete(WebdavStore store, ResourceLocks resourceLocks, boolean readOnly, int debug) {
        this.store = store;
        this.resourceLocks = resourceLocks;
        this.readOnly = readOnly;
        this.debug = debug;
    }

    public void execute(Transaction transaction, HttpServletRequest req, HttpServletResponse resp) throws IOException {
        if (debug == 1)
            System.err.println("-- doDelete");

        if (!readOnly) {
            String path = getRelativePath(req);
            String lockOwner = "doDelete" + System.currentTimeMillis()
                    + req.toString();
            if (resourceLocks.lock(path, lockOwner, true, -1)) {
                try {
                    Hashtable errorList = new Hashtable();
                    deleteResource(transaction, path, errorList, req, resp);
                    if (!errorList.isEmpty()) {
                        sendReport(req, resp, errorList);
                    }
                } catch (AccessDeniedException e) {
                    resp.sendError(WebdavStatus.SC_FORBIDDEN);
                } catch (ObjectAlreadyExistsException e) {
                    resp.sendError(WebdavStatus.SC_NOT_FOUND, req
                            .getRequestURI());
                } catch (WebdavException e) {
                    resp.sendError(WebdavStatus.SC_INTERNAL_SERVER_ERROR);
                } finally {
                    resourceLocks.unlock(path, lockOwner);
                }
            } else {
                resp.sendError(WebdavStatus.SC_INTERNAL_SERVER_ERROR);
            }
        } else {
            resp.sendError(WebdavStatus.SC_FORBIDDEN);
        }

    }

    /**
     * deletes the recources at "path"
     *
     * @param path
     *            the folder to be deleted
     * @param errorList
     *            all errors that ocurred
     * @param req
     *            HttpServletRequest
     * @param resp
     *            HttpServletResponse
     * @throws WebdavException
     *             if an error in the underlying store occurs
     * @throws IOException
     *             when an error occurs while sending the response
     */
    public void deleteResource(Transaction transaction, String path, Hashtable errorList,
            HttpServletRequest req, HttpServletResponse resp)
            throws IOException, WebdavException {

        resp.setStatus(WebdavStatus.SC_NO_CONTENT);
        if (!readOnly) {

            if (store.isResource(transaction, path)) {
                store.removeObject(transaction, path);
            } else {
                if (store.isFolder(transaction, path)) {

                    deleteFolder(transaction, path, errorList, req, resp);
                    store.removeObject(transaction, path);
                } else {
                    resp.sendError(WebdavStatus.SC_NOT_FOUND);
                }
            }

        } else {
            resp.sendError(WebdavStatus.SC_FORBIDDEN);
        }
    }


    /**
     *
     * helper method of deleteResource() deletes the folder and all of its
     * contents
     *
     * @param path
     *            the folder to be deleted
     * @param errorList
     *            all errors that ocurred
     * @param req
     *            HttpServletRequest
     * @param resp
     *            HttpServletResponse
     * @throws WebdavException
     *             if an error in the underlying store occurs
     */
    private void deleteFolder(Transaction transaction, String path, Hashtable errorList,
            HttpServletRequest req, HttpServletResponse resp)
            throws WebdavException {

        String[] children = store.getChildrenNames(transaction, path);
        for (int i = children.length - 1; i >= 0; i--) {
            children[i] = "/" + children[i];
            try {
                if (store.isResource(transaction, path + children[i])) {
                    store.removeObject(transaction, path + children[i]);

                } else {
                    deleteFolder(transaction, path + children[i], errorList, req, resp);

                    store.removeObject(transaction, path + children[i]);

                }
            } catch (AccessDeniedException e) {
                errorList.put(path + children[i], new Integer(
                        WebdavStatus.SC_FORBIDDEN));
            } catch (ObjectNotFoundException e) {
                errorList.put(path + children[i], new Integer(
                        WebdavStatus.SC_NOT_FOUND));
            } catch (WebdavException e) {
                errorList.put(path + children[i], new Integer(
                        WebdavStatus.SC_INTERNAL_SERVER_ERROR));
            }
        }

    }


}
