package net.sf.webdav.methods;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.DocumentBuilder;

import net.sf.webdav.ResourceLocks;
import net.sf.webdav.Transaction;
import net.sf.webdav.WebdavStatus;
import net.sf.webdav.WebdavStore;
import net.sf.webdav.exceptions.AccessDeniedException;
import net.sf.webdav.exceptions.WebdavException;
import net.sf.webdav.fromcatalina.XMLHelper;
import net.sf.webdav.fromcatalina.XMLWriter;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;

public class DoProppatch extends AbstractMethod {
    
    private int debug;
    private boolean readOnly;
    private WebdavStore store;
    private ResourceLocks resLocks;
    


    public DoProppatch(WebdavStore store, ResourceLocks resLocks, boolean readOnly, int debug) {
        this.debug = debug;
        this.readOnly = readOnly;
        this.store = store;
        this.resLocks = resLocks;
    }

    public void execute(Transaction transaction, HttpServletRequest req, HttpServletResponse resp) throws IOException {
        if (debug == 1)
            System.err.println("-- doProppatch");

        if (readOnly) {
            resp.sendError(WebdavStatus.SC_FORBIDDEN);
            return;
        }
        
        // TODO for now, PROPPATCH just sends a valid response, stating that
        // everything is fine, but doesn't do anything.
        
        
        // Retrieve the resources
        String lockOwner = "doPropfind" + System.currentTimeMillis()
                + req.toString();
        String path = getRelativePath(req);

        if (resLocks.lock(path, lockOwner, false, 0)) {
            try {
                if (!store.objectExists(transaction, path)) {
                    resp.sendError(HttpServletResponse.SC_NOT_FOUND);
                    return;
                    // we do not to continue since there is no root
                    // resource
                }

                Vector toset = null;
                Vector toremove = null;
                Vector tochange = new Vector(); // contains all properties from toset and toremove
                
                path = getCleanPath(getRelativePath(req));
                

                Node tosetNode = null;
                Node toremoveNode = null;

                if (req.getContentLength() != 0) {
                    DocumentBuilder documentBuilder = getDocumentBuilder();
                    try {
                        Document document = documentBuilder.parse(new InputSource(req
                                .getInputStream()));
                        // Get the root element of the document
                        Element rootElement = document.getDocumentElement();
                        
                        tosetNode = XMLHelper.findSubElement(XMLHelper.findSubElement(
                            rootElement, "set"), "prop");
                        toremoveNode = XMLHelper.findSubElement(XMLHelper.findSubElement(
                            rootElement, "remove"), "prop");
                    } catch (Exception e) {
                        resp.sendError(WebdavStatus.SC_INTERNAL_SERVER_ERROR);
                        return;
                    }
                } else {
                    // no content: error
                    resp.sendError(WebdavStatus.SC_INTERNAL_SERVER_ERROR);
                    return;
                }
                
                HashMap namespaces = new HashMap();
                namespaces.put("DAV:", "D");
                
                if (tosetNode != null) {
                    toset = XMLHelper.getPropertiesFromXML(tosetNode);
                    tochange.addAll(toset);
                }

                if (toremoveNode != null) {
                    toremove = XMLHelper.getPropertiesFromXML(toremoveNode);
                    tochange.addAll(toremove);
                }

                resp.setStatus(WebdavStatus.SC_MULTI_STATUS);
                resp.setContentType("text/xml; charset=UTF-8");
                resp.setCharacterEncoding("UTF-8");

                // Create multistatus object
                XMLWriter generatedXML = new XMLWriter(resp.getWriter(), namespaces);
                generatedXML.writeXMLHeader();
                generatedXML.writeElement("DAV::multistatus", XMLWriter.OPENING);
                
                generatedXML.writeElement("DAV::response", XMLWriter.OPENING);
                String status = new String("HTTP/1.1 " + WebdavStatus.SC_OK + " "
                        + WebdavStatus.getStatusText(WebdavStatus.SC_OK));

                // Generating href element
                generatedXML.writeElement("DAV::href", XMLWriter.OPENING);

                String href = req.getContextPath();
                if ((href.endsWith("/")) && (path.startsWith("/")))
                    href += path.substring(1);
                else
                    href += path;
                if ((store.isFolder(transaction, path)) && (!href.endsWith("/")))
                    href += "/";

                generatedXML.writeText(rewriteUrl(href));

                generatedXML.writeElement("DAV::href", XMLWriter.CLOSING);
                
                for (Iterator iter = tochange.iterator(); iter.hasNext();) {
                    String property = (String)iter.next();
                    
                    generatedXML.writeElement("DAV::propstat", XMLWriter.OPENING);
                    
                    generatedXML.writeElement("DAV::prop", XMLWriter.OPENING);
                    generatedXML.writeElement(property, XMLWriter.NO_CONTENT);
                    generatedXML.writeElement("DAV::prop", XMLWriter.CLOSING);

                    generatedXML.writeElement("DAV::status", XMLWriter.OPENING);
                    generatedXML.writeText(status);
                    generatedXML.writeElement("DAV::status", XMLWriter.CLOSING);
                    
                    generatedXML.writeElement("DAV::propstat", XMLWriter.CLOSING);
                }
                
                generatedXML.writeElement("DAV::response", XMLWriter.CLOSING);

                generatedXML.writeElement("DAV::multistatus", XMLWriter.CLOSING);
                
                if (debug == 1) {
                    System.out.println(generatedXML.toString());
                }
                generatedXML.sendData();
            } catch (AccessDeniedException e) {
                resp.sendError(WebdavStatus.SC_FORBIDDEN);
            } catch (WebdavException e) {
                resp.sendError(WebdavStatus.SC_INTERNAL_SERVER_ERROR);
            } catch (ServletException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            } finally {
                resLocks.unlock(path, lockOwner);
            }
        } else {
            resp.sendError(WebdavStatus.SC_INTERNAL_SERVER_ERROR);
        }
    }
}
